#pragma once

#include <map>
#include <memory>
#include <string>

#include "ProviderSegment.h"
#include "detail/TypedEntityContainer.h"


namespace armarx::armem
{

    class CoreSegment;
    using CoreSegmentPtr = std::unique_ptr<CoreSegment>;

    /**
     * @brief Data of a core segment containing multiple provider segments.
     */
    class CoreSegment : public detail::TypedEntityContainer<ProviderSegmentPtr, CoreSegment>
    {
        using Base = detail::TypedEntityContainer<ProviderSegmentPtr, CoreSegment>;

    public:

        CoreSegment();
        CoreSegment(const std::string& name, aron::type::AronTypePtr aronType = nullptr);
        CoreSegment(const std::string& name, const MemoryID& parentID, aron::type::AronTypePtr aronType = nullptr);
        CoreSegment(const MemoryID& id, aron::type::AronTypePtr aronType = nullptr);

        CoreSegment(const CoreSegment& other);
        CoreSegment& operator=(const CoreSegment& other);

        ~CoreSegment() override;


        using Base::name;
        const std::string& name() const override;


        bool hasProviderSegment(const std::string& name) const;
        ProviderSegment& getProviderSegment(const std::string& name);
        const ProviderSegment& getProviderSegment(const std::string& name) const;


        using Base::getEntity;
        const Entity& getEntity(const MemoryID& id) const override;


        using Base::update;
        virtual MemoryID update(const InternalEntityUpdate& update) override;

        /**
         * @brief Add an empty provider segment with the given name and optional provider segment type.
         * @param name The segment name.
         * @param providerSegmentType The provider type. If nullptr, the core segment type is used.
         * @return The added provider segment.
         */
        ProviderSegment& addProviderSegment(const std::string& name, aron::type::AronTypePtr providerSegmentType = nullptr);
        /// Copy and insert a provider segment.
        ProviderSegment& addProviderSegment(const ProviderSegment& providerSegment);
        /// Move and insert a provider segment.
        ProviderSegment& addProviderSegment(ProviderSegmentPtr&& providerSegment);


        // MemoryItem interface
    public:
        std::string getLevelName() const override;

        // MemoryContainer interface
    protected:
        ContainerT& _underlyingContainer() override;
        void _copyElements(CoreSegment& other, bool withData) const override;


    public:

        /// The provider segments.
        std::map<std::string, ProviderSegmentPtr> providerSegments;

    };

}
