#pragma once

#include <RobotAPI/interface/armem/query.h>

#include "../memory/Memory.h"
#include "../memory/ice_conversions.h"

#include "BaseQueryProcessor.h"


namespace armarx::armem
{

    /**
     * @brief Handles memory queries.
     */
    class EntityQueryProcessor : public BaseQueryProcessor<Entity, query::EntityQuery>
    {
    public:

        using BaseQueryProcessor::process;

        void process(Entity& result, const query::EntityQuery& query, const Entity& entity) const override;

        void process(Entity& result, const query::entity::All& query, const Entity& entity) const;
        void process(Entity& result, const query::entity::Single& query, const Entity& entity) const;
        void process(Entity& result, const query::entity::TimeRange& query, const Entity& entity) const;
        void process(Entity& result, const query::entity::IndexRange& query, const Entity& entity) const;

        void process(Entity& result, const Time& min, const Time& max, const Entity& entity,
                     const query::EntityQuery& query) const;


        data::EntityPtr processToIce(const query::EntityQuery& query, const Entity& entity) const;


        static size_t negativeIndexSemantics(long index, size_t size);

    private:

        void addResultSnapshot(Entity& result, Entity::ContainerT::const_iterator it, const query::EntityQuery& query) const;
        void addResultSnapshot(Entity& result, const EntitySnapshot& snapshot, const query::EntityQuery& query) const;

    };

}
