/*
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @package    RobotAPI::ArmarXObjects::armem
 * @author     Simon Ottenhaus ( simon dot ottenhaus at kit dot edu )
 * @date       2020
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#define BOOST_TEST_MODULE RobotAPI::ArmarXLibraries::armem

#define ARMARX_BOOST_TEST

#include <RobotAPI/Test.h>
#include "../client/query/QueryBuilder.h"
#include "../error.h"

#include <iostream>


namespace armem = armarx::armem;
namespace aron = armarx::aron;


namespace ArMemQueryBuilderTest
{
    struct Fixture
    {
        Fixture()
        {
        }
        ~Fixture()
        {
        }
    };
}


BOOST_FIXTURE_TEST_SUITE(ArMemQueryBuilderTest, ArMemQueryBuilderTest::Fixture)



BOOST_AUTO_TEST_CASE(test_all_all)
{
    armem::client::QueryBuilder qb;
    qb
    .coreSegments().all()
    .providerSegments().all()
    .entities().all()
    .snapshots().all();

    std::vector<armem::query::MemoryQueryPtr> memoryQueries = qb.finalize();

    BOOST_REQUIRE_EQUAL(memoryQueries.size(), 1);
    armem::query::MemoryQueryPtr memoryQuery = memoryQueries.front();
    BOOST_CHECK(armem::query::memory::AllPtr::dynamicCast(memoryQuery));

    BOOST_REQUIRE_EQUAL(memoryQuery->coreSegmentQueries.size(), 1);
    armem::query::CoreSegmentQueryPtr coreSegQuery = memoryQuery->coreSegmentQueries.front();
    BOOST_CHECK(armem::query::core::AllPtr::dynamicCast(coreSegQuery));

    BOOST_REQUIRE_EQUAL(coreSegQuery->providerSegmentQueries.size(), 1);
    armem::query::ProviderSegmentQueryPtr provSegQuery = coreSegQuery->providerSegmentQueries.front();
    BOOST_CHECK(armem::query::provider::AllPtr::dynamicCast(provSegQuery));

    BOOST_REQUIRE_EQUAL(provSegQuery->entityQueries.size(), 1);
    armem::query::EntityQueryPtr entityQuery = provSegQuery->entityQueries.front();
    BOOST_CHECK(armem::query::entity::AllPtr::dynamicCast(entityQuery));
}





BOOST_AUTO_TEST_SUITE_END()
