#include "InstanceTreeWidget.h"

#include <QHeaderView>
#include <QLabel>

#include <SimoxUtility/algorithm/string.h>

#include <ArmarXCore/core/exceptions/local/ExpressionException.h>

#include <RobotAPI/libraries/aron/aroncore/navigators/typenavigator/AronObjectTypeNavigator.h>
#include <RobotAPI/libraries/armem_gui/QTreeWidgetAronDataVisitor.h>


namespace armarx::armem
{

    InstanceTreeWidget::InstanceTreeWidget()
    {
        initWidget();
    }

    void InstanceTreeWidget::initWidget()
    {
        clear();
        QStringList columns;
        columns.insert(int(Columns::KEY), "Key");
        columns.insert(int(Columns::TYPE), "Type");
        columns.insert(int(Columns::VALUE), "Value");
        setColumnCount(columns.size());
        setHeaderLabels(columns);

        // header()->setMinimumSectionSize(25);
        // header()->resizeSection(int(Columns::KEY), 250);
        // header()->setTextElideMode(Qt::TextElideMode::ElideRight);
    }

    void InstanceTreeWidget::update(const MemoryID& id, const Memory& memory)
    {
        const armem::EntityInstance* instance;
        try
        {
            instance = &memory.getEntityInstance(id);
        }
        catch (const armem::error::ArMemError& e)
        {
            if (statusLabel)
            {
                statusLabel->setText(QString::fromStdString(e.what()));
            }
            return;
        };
        if (instance)
        {
            update(*instance);
        }
    }

    void InstanceTreeWidget::update(const EntityInstance& instance)
    {
        QTreeWidgetItem* item = new QTreeWidgetItem();
        item->setText(0, QString::fromStdString(instance.id().str()));

        if (!instance.data())
        {
            item->setText(0, "(No data.)");
        }
        else if (!instance.dataNavigator())
        {
            item->setText(0, "(No introspection available.)");
        }
        else
        {
            armem::QTreeWidgetAronDataVisitor visitor(item);
            visitor.applyTo(*instance.dataNavigator());
        }

        clear();
        addTopLevelItem(item);

        item->setExpanded(true);
        emit updated();
    }

    void InstanceTreeWidget::setStatusLabel(QLabel* statusLabel)
    {
        this->statusLabel = statusLabel;
    }

}
