#include "QTreeWidgetAronDataVisitor.h"

#include <ArmarXCore/core/exceptions/local/ExpressionException.h>


namespace armarx::armem
{

    QTreeWidgetAronDataVisitor::QTreeWidgetAronDataVisitor()
    {
    }

    QTreeWidgetAronDataVisitor::QTreeWidgetAronDataVisitor(QTreeWidgetItem* root)
    {
        rootItems.push(root);
    }

    QTreeWidgetAronDataVisitor::~QTreeWidgetAronDataVisitor()
    {}

    bool QTreeWidgetAronDataVisitor::_visitEnter(const std::string& key, const std::string& type, size_t numChildren)
    {
        QTreeWidgetItem* item = nullptr;
        if (rootItems.size() > 0)
        {
            item = rootItems.top();
            rootItems.pop();
        }
        else
        {
            QStringList cols;
            cols.insert(0, QString::fromStdString(key));
            cols.insert(1, QString::fromStdString(type));
            cols.insert(2, QString::number(numChildren) + " items");
            item = new QTreeWidgetItem(cols);
        }
        items.push(item);
        return true;
    }

    bool QTreeWidgetAronDataVisitor::_visitExit()
    {
        ARMARX_CHECK_POSITIVE(items.size());
        QTreeWidgetItem* item = items.top();
        items.pop();
        item->setExpanded(true);
        if (items.size() > 0)
        {
            items.top()->addChild(item);
        }
        else
        {
            rootItems.push(item);
        }
        return true;
    }

    void QTreeWidgetAronDataVisitor::streamValueText(AronStringDataNavigator& n, std::stringstream& ss)
    {
        ss << "'" << n.getValue() << "'";
    }

    void QTreeWidgetAronDataVisitor::streamValueText(AronNDArrayDataNavigator& n, std::stringstream& ss)
    {
        ss << "shape (" << simox::alg::join(simox::alg::multi_to_string(n.getDimensions()), ", ") << ")";
    }


}
