#pragma once

#include <sstream>
#include <stack>

#include <QTreeWidget>
#include <QLabel>

#include <RobotAPI/libraries/aron/aroncore/navigators/visitors/AronDataVisitor.h>


namespace armarx::armem
{



    struct QTreeWidgetAronDataVisitor : public aron::visitor::AronDataVisitor
    {
        std::stack<QTreeWidgetItem*> rootItems;
        std::stack<QTreeWidgetItem*> items;


        QTreeWidgetAronDataVisitor();
        QTreeWidgetAronDataVisitor(QTreeWidgetItem* root);
        virtual ~QTreeWidgetAronDataVisitor() override;


        bool visitEnter(const std::string& key, AronDictDataNavigator& n) override
        {
            return _visitEnter(key, "Dict", n.childrenSize());
        }
        bool visitExit(AronDictDataNavigator&) override
        {
            return _visitExit();
        }

        bool visitEnter(const std::string& key, AronListDataNavigator& n) override
        {
            return _visitEnter(key, "List", n.childrenSize());
        }
        bool visitExit(AronListDataNavigator&) override
        {
            return _visitExit();
        }

        // Same for Dict and List
        bool _visitEnter(const std::string& key, const std::string& type, size_t numChildren);
        bool _visitExit();


        bool visit(const std::string& key, AronBoolDataNavigator& b) override
        {
            this->addValueRow(key, b, "Bool");
            return true;
        }
        bool visit(const std::string& key, AronDoubleDataNavigator& d) override
        {
            this->addValueRow(key, d, "Double");
            return true;
        }
        bool visit(const std::string& key, AronFloatDataNavigator& f) override
        {
            this->addValueRow(key, f, "Float");
            return true;
        }
        bool visit(const std::string& key, AronIntDataNavigator& i) override
        {
            this->addValueRow(key, i, "Int");
            return true;
        }
        bool visit(const std::string& key, AronLongDataNavigator& l) override
        {
            this->addValueRow(key, l, "Long");
            return true;
        }
        bool visit(const std::string& key, AronStringDataNavigator& string) override
        {
            this->addValueRow(key, string, "String");
            return true;
        }

        bool visit(const std::string& key, AronNDArrayDataNavigator& array) override
        {
            this->addValueRow(key, array, "ND Array");
            return true;
        }


    private:

        template <class Navigator>
        void addValueRow(const std::string& key, Navigator& n, const std::string& typeName)
        {
            if (items.size() > 0)
            {
                items.top()->addChild(new QTreeWidgetItem(this->makeValueRowStrings(key, n, typeName)));
            }
        }

        template <class Navigator>
        QStringList makeValueRowStrings(const std::string& key, Navigator& n, const std::string& typeName)
        {
            std::stringstream value;
            streamValueText(n, value);
            return
            {
                QString::fromStdString(key),
                QString::fromStdString(typeName),
                QString::fromStdString(value.str()),
            };
        }

        template <class Navigator>
        void streamValueText(Navigator& n, std::stringstream& ss)
        {
            ss << n.getValue();
        }
        void streamValueText(AronStringDataNavigator& n, std::stringstream& ss);
        void streamValueText(AronNDArrayDataNavigator& n, std::stringstream& ss);

    };


}
