/*
 * This file is part of ArmarX.
 *
 * Copyright (C) 2012-2016, High Performance Humanoid Technologies (H2T),
 * Karlsruhe Institute of Technology (KIT), all rights reserved.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Fabian Peller-Konrad (fabian dot peller-konrad at kit dot edu)
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

// STD/STL

// Header
#include "AronDataNavigatorFactory.h"

namespace armarx
{
    namespace aron
    {
        namespace datanavigator
        {

            // Map types to factories
            const std::map<AronDataDescriptor, AronDataNavigatorFactoryPtr> AronDataNavigatorFactory::FACTORIES =
            {
#define RUN_ARON_MACRO(upperType, lowerType, capsType) \
    {AronResolver::GetTypeDescriptorForAronDataId(typeid(data::Aron##upperType)), AronDataNavigatorFactoryPtr(new Aron##upperType##DataNavigatorFactory())}, \

                HANDLE_CONTAINER_DATA
                HANDLE_COMPLEX_DATA
                HANDLE_PRIMITIVE_TYPES
#undef RUN_ARON_MACRO
            };

            // Access method
            AronDataNavigatorPtr AronDataNavigatorFactory::create(const data::AronDataPtr& aron, const AronPath& path) const
            {
                CheckIfInputIsNull("AronDataNavigatorFactory", "create", path, aron);

                auto typeDescriptor = AronResolver::GetTypeDescriptorForAronData(aron);
                auto factory_iterator = FACTORIES.find(typeDescriptor);
                if (factory_iterator == FACTORIES.end())
                {
                    throw exception::AronExceptionWithPathInfo("AronDataNavigatorFactory", "create", "Cannot find the desired factory for input: " + AronDataDescriptorToString.at(typeDescriptor) + ". Cannot create navigator", path);
                }
                return factory_iterator->second->createSpecific(aron, path);
            }

            AronDataNavigatorPtr AronDataNavigatorFactory::createSpecific(const data::AronDataPtr&, const AronPath& p) const
            {
                throw exception::AronExceptionWithPathInfo("AronDataNavigatorFactory", "createSpecific", "Called disallowed method of an AronDataNavigatorFactory. Use child class instead!", p);
            }

            // Factories
#define RUN_ARON_MACRO(upperType, lowerType, capsType) \
    AronDataNavigatorPtr Aron##upperType##DataNavigatorFactory::createSpecific(const data::AronDataPtr& aron, const AronPath& path) const \
    { \
        CheckIfInputIsNull("Aron"+std::string(#upperType)+"DataNavigatorFactory", "create", path, aron); \
        data::Aron##upperType##Ptr aronCasted = data::Aron##upperType##Ptr::dynamicCast(aron); \
        CheckIfInputIsNull("Aron"+std::string(#upperType)+"DataNavigatorFactory", "create", path, aronCasted); \
        return AronDataNavigatorPtr(new Aron##upperType##DataNavigator(aronCasted, path)); \
    }

            HANDLE_CONTAINER_DATA
            HANDLE_COMPLEX_DATA
            HANDLE_PRIMITIVE_TYPES
#undef RUN_ARON_MACRO

        }
    }
}
